# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: frames.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x66rames.proto\x12\x06\x61pibtc\"\x15\n\x04UUID\x12\r\n\x05Value\x18\x01 \x01(\x0c\"\x1a\n\tTimestamp\x12\r\n\x05Value\x18\x01 \x01(\x03\"\x1a\n\tSignature\x12\r\n\x05Value\x18\x01 \x01(\x0c\"\x1a\n\tPublicKey\x12\r\n\x05Value\x18\x01 \x01(\x0c\"|\n\x0f\x41uthTokenHeader\x12\x1d\n\x07TokenId\x18\x01 \x01(\x0b\x32\x0c.apibtc.UUID\x12$\n\tPublicKey\x18\x02 \x01(\x0b\x32\x11.apibtc.PublicKey\x12$\n\tTimestamp\x18\x03 \x01(\x0b\x32\x11.apibtc.Timestamp\"Z\n\tAuthToken\x12\'\n\x06Header\x18\x01 \x01(\x0b\x32\x17.apibtc.AuthTokenHeader\x12$\n\tSignature\x18\x02 \x01(\x0b\x32\x11.apibtc.SignatureB\t\xaa\x02\x06\x41piBtcb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'frames_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\006ApiBtc'
  _UUID._serialized_start=24
  _UUID._serialized_end=45
  _TIMESTAMP._serialized_start=47
  _TIMESTAMP._serialized_end=73
  _SIGNATURE._serialized_start=75
  _SIGNATURE._serialized_end=101
  _PUBLICKEY._serialized_start=103
  _PUBLICKEY._serialized_end=129
  _AUTHTOKENHEADER._serialized_start=131
  _AUTHTOKENHEADER._serialized_end=255
  _AUTHTOKEN._serialized_start=257
  _AUTHTOKEN._serialized_end=347
# @@protoc_insertion_point(module_scope)
